@@ -8,6 +8,7 @@ from django.conf import settings
 from lino.utils import format_date
 from lino.utils.media import TmpMediaFile
 from lino.core import actions
+from lino.core.keyboard import Hotkey
 from lino.core.roles import Expert
 from lino.api import dd, rt, _
 
@@ -32,6 +33,7 @@ class PrintTableAction(actions.Action):
     combo_group = 'pdf'
     callable_from = 't'
     required_roles = dd.login_required(Expert)
+    hotkey = Hotkey('p', ctrl=True)
 
     # def is_callable_from(self, caller):
     #     return isinstance(caller, actions.ShowTable)
