@@ -113,20 +113,28 @@ class Entry(UserAuthored, Controllable, CombinedDateTime,
     def preview(self, ar):
         if ar is None:
             return
+        return self.render_from(self.publisher_page_template, ar)
 
-        env = settings.SITE.plugins.jinja.renderer.jinja_env
-        template = env.get_template(self.publisher_page_template)
+    def get_item_preview(self, ar):
+        if ar is None:
+            return
+        return self.render_from(self.publisher_item_template, ar)
 
-        context = self.get_preview_context(ar)
+    @dd.htmlbox()
+    def item_preview(self, ar):
+        return self.get_item_preview(ar)
 
-        return template.render(**context)
+    def as_summary_row(self, ar):
+        return self.get_item_preview(ar)
 
     def get_preview_context(self, ar):
         context = super().get_preview_context(ar)
         FileUsage = rt.models.albums.FileUsage
-        primary_image = FileUsage.objects.filter(
-            **gfk2lookup(FileUsage.owner, self), primary_image=True)[0].file
-        context.update(pi=primary_image)
+        qs = FileUsage.objects.filter(
+            **gfk2lookup(FileUsage.owner, self), primary_image=True)
+        if qs.count() == 1:
+            primary_image = qs[0].file
+            context.update(pi=primary_image)
 
         if ar.__class__ == BaseRequest:
             return context
@@ -214,7 +222,7 @@ class Entries(dd.Table):
     required_roles = set([BlogsReader])
 
     model = 'blogs.Entry'
-    column_names = "id pub_date user entry_type title * body"
+    column_names = "id pub_date user entry_type title *"
     order_by = ["-id"]
 
     hide_top_toolbar = False
@@ -246,6 +254,10 @@ class PublicEntries(Entries):
     comments.CommentsByRFC
     """
 
+    display_mode = constants.DISPLAY_MODE_STORY
+
+    item_layout = "item_preview"
+
     @classmethod
     def collect_extra_actions(cls):
         return []
@@ -317,13 +329,12 @@ class EntriesByController(Entries):
 
 
 
-class LatestEntries(Entries):
+class LatestEntries(PublicEntries):
     """Show the most recent blog entries."""
     label = _("Latest blog entries")
     column_names = "pub_date title user *"
     order_by = ["-pub_date"]
     filter = models.Q(pub_date__isnull=False)
-    display_mode = 'summary'
 
     @classmethod
     def get_table_summary(cls, obj, ar, max_num=10, **context):
