@@ -586,15 +586,27 @@ class RecurrentEvents(dd.Table):
 
 
 class EventDetail(dd.DetailLayout):
+
     start = "start_date start_time"
     end = "end_date end_time"
-    main = """
-    event_type summary user
-    start end #all_day assigned_to #duration #state
+    notify = "notify_before notify_unit"
+
+    general_base = dd.Panel("""
+    event_type summary notify
+    start end #all_day #duration #state
     room project owner workflow_buttons
     # owner created:20 modified:20
-    description GuestsByEvent #outbox.MailsByController
-    """
+    description #outbox.MailsByController
+    """, label=_("General"))
+
+    guests = dd.Panel("""
+    user assigned_to
+    GuestsByEvent
+    """, label=_("Presences"))
+
+    main = "general_base guests"
+
+
 
 class EventInsert(dd.InsertLayout):
     main = """
