@@ -71,14 +71,8 @@ class Topics(dd.Table):
     # id
     # """
 
-
-class Lines(dd.Table):
-    model = 'courses.Line'
-    required_roles = dd.login_required(CoursesUser)
-    column_names = ("ref name topic course_area "
-                    "event_type guest_role every_unit every *")
-    order_by = ['ref', 'name']
-    detail_layout = """
+class LineDetail(dd.DetailLayout):
+    main = """
     id name ref
     company contact_person
     course_area topic fees_cat fee options_cat body_template
@@ -87,6 +81,14 @@ class Lines(dd.Table):
     description
     # courses.ActivitiesByLine
     """
+
+class Lines(dd.Table):
+    model = 'courses.Line'
+    required_roles = dd.login_required(CoursesUser)
+    column_names = ("ref name topic course_area "
+                    "event_type guest_role every_unit every *")
+    order_by = ['ref', 'name']
+    detail_layout = "courses.LineDetail"
     insert_layout = dd.InsertLayout("""
     name
     ref topic
@@ -151,8 +153,6 @@ class CourseDetail(dd.DetailLayout):
     EnrolmentsByCourse
     """, label=_("Enrolments"))
 
-# Course.detail_layout_class = CourseDetail
-# print(20201223, ActivityLayouts.get_list_items())
 
 class Activities(dd.Table):
     activity_layout = None
