@@ -707,7 +707,7 @@ def setup_memo_commands(sender=None, **kwargs):
 
     Course = sender.models.courses.Course
 
-    def cmd(parser, s):
+    def cmd(parser, s, cmdname, usages):
 
         pk = s
         txt = None
