@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2016-2020 Rumma & Ko Ltd
+# Copyright 2016-2021 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
 
 
@@ -23,16 +23,27 @@ from lino.utils import ONE_DAY
 from lino.api import dd, rt, _
 from lino_xl.lib.ledger.utils import ZERO
 from lino_xl.lib.ledger.roles import LedgerUser, LedgerStaff
+# from lino_xl.lib.cal.choicelists import DurationUnits
 
 from .mixins import InvoiceGenerator
 # from .choicelists import InvoicingCycles
 # from .choicelists import InvoicingDepartments
 from .actions import (ToggleSelection, StartInvoicing,
                       # StartInvoicingByArea,
-                      StartInvoicingForPartner, ExecutePlan,
-                      ExecuteItem)
+                      StartInvoicingForPartner, StartInvoicingForOrder,
+                      ExecutePlan, ExecuteItem)
 
 
+order_model = dd.plugins.invoicing.order_model
+invoiceable_label = dd.plugins.invoicing.invoiceable_label
+generator_label = _("Generator")
+
+# if dd.is_installed('invoicing'):
+#     order_model = dd.plugins.invoicing.order_model
+# else:
+#     order_model = None
+#
+
 class SalesRule(dd.Model):
     class Meta:
         app_label = 'invoicing'
@@ -57,6 +68,10 @@ def create_salesrule(sender, instance, created, **kwargs):
         if not hasattr(instance, 'salesrule'):
             rt.models.invoicing.SalesRule.objects.create(partner=instance)
 
+def get_invoice_recipient(p):
+    if hasattr(p, 'salesrule'):
+        return p.salesrule.invoice_recipient or p
+    return p
 
 
 class SalesRules(dd.Table):
@@ -84,6 +99,26 @@ dd.inject_action(
         PartnersByInvoiceRecipient))
 
 
+# class SubscriptionType(BabelDesignated):
+#     class Meta(object):
+#         app_label = 'invoicing'
+#         abstract = dd.is_abstract_model(__name__, 'SubscriptionType')
+#         verbose_name = _("Subscription type")
+#         verbose_name_plural = _("Subscription types")
+#
+#     renew_every = models.IntegerField(_("Renew every"), default=1)
+#     renew_by = DurationUnits.field(_("Renew by"), blank=True, null=True)
+#     renew_before = models.IntegerField(_("Renew before"), default=0)
+#
+#
+# class SubscriptionTypes(dd.Table):
+#     required_roles = dd.login_required(LedgerUser)
+#     model = "invoicing.SubscriptionType"
+#     column_names = "designation renew_every renew_by *"
+#     order_by = ['designation']
+
+
+
 class Tariff(BabelDesignated):
 
     class Meta(object):
@@ -124,7 +159,6 @@ class Tariffs(dd.Table):
     order_by = ['designation']
 
 
-#
 class Area(BabelDesignated, Sequenced):
     class Meta:
         app_label = 'invoicing'
@@ -161,8 +195,6 @@ class Areas(dd.Table):
     column_names = "seqno designation journal *"
 
 
-
-
 class Plan(UserPlan):
     class Meta:
         app_label = 'invoicing'
@@ -171,18 +203,20 @@ class Plan(UserPlan):
         verbose_name_plural = _("Invoicing plans")
 
     area = dd.ForeignKey('invoicing.Area', blank=True)
-    min_date = models.DateField(
-        _("Invoiceables from"), null=True, blank=True)
-    max_date = models.DateField(
-        _("until"), null=True, blank=True)
+    min_date = models.DateField(_("Invoiceables from"), null=True, blank=True)
+    max_date = models.DateField(_("until"), null=True, blank=True)
+    order = dd.ForeignKey(order_model, blank=True, null=True)
     partner = dd.ForeignKey('contacts.Partner', blank=True, null=True)
 
-    execute_plan = ExecutePlan()
     start_plan = StartInvoicing()
+    execute_plan = ExecutePlan()
 
     def full_clean(self):
         if self.area_id is None:
             self.area = rt.models.invoicing.Area.objects.first()
+            if self.area is None:
+                raise Warning(
+                    _("Please configure at least one invoicing area."))
         super(Plan, self).full_clean()
 
     def get_max_date(self):
@@ -193,6 +227,7 @@ class Plan(UserPlan):
     def get_generators_for_plan(self, partner=None):
         for m in rt.models_by_base(InvoiceGenerator):
             for obj in m.get_generators_for_plan(self, partner):
+                # print("20210727", obj)
                 # if obj.get_invoiceable_product(self) is not None:
                 yield obj
 
@@ -214,14 +249,13 @@ class Plan(UserPlan):
                 continue
                 # raise Exception("{!r} has no invoice recipient".format(
                 #     ig))
-            if hasattr(partner, 'salesrule'):
-                partner = partner.salesrule.invoice_recipient or partner
 
+            partner = get_invoice_recipient(partner)
             invoice = self.create_invoice(
                 partner=partner, user=ar.get_user())
 
             # dd.logger.info("20181114 b", obj)
-            info = ig.compute_invoicing_info(max_date)
+            info = ig.compute_invoicing_info(self.min_date, max_date)
             # if not info.invoiceable_product:
             #     continue
             # print("20200425", ig, info, invoice)
@@ -231,58 +265,28 @@ class Plan(UserPlan):
                 # dd.logger.debug("20181126 no invoice items for %s", ig)
                 continue
 
+            # print("20210731", partner, invoice_items)
+
+            # assert invoice.pk is None
+            # for i in invoice_items:
+            #     assert i.pk is None
+
+            # print("20210731 collect", self, ig, collected)
             if ig.allow_group_invoices():
                 item = collected.get(partner.pk, None)
                 if item is None:
-                    item = Item(plan=self, partner=partner)
+                    item = Item(plan=self, partner=partner, preview="")
                     collected[partner.pk] = item
+                    # item.preview = ""
             else:
-                item = Item(plan=self, partner=partner, generator=ig)
+                item = Item(plan=self, partner=partner, generator=ig, preview="")
                 # collected[obj] = item
+                # item.preview = ""
 
-                # gfk = self._meta.get_field('owner')
-                # kwargs = gfk2lookup(gfk, self, **kwargs)
-                # return model.objects.filter(**kwargs)
-
-            # for ie in info.used_events:
-            # # for ie in obj.get_invoiceable_events(
-            # #         self.today, self.max_date):
-            #     idate = ig.get_invoiceable_event_date(ie)
-            #     if idate is None:
-            #         dd.logger.info("20181113 idate is None")
-            #         continue
-            #     if item.first_date is None:
-            #         item.first_date = idate
-            #     else:
-            #         item.first_date = min(idate, item.first_date)
-            #     if item.last_date is None:
-            #         item.last_date = idate
-            #     else:
-            #         item.last_date = max(idate, item.last_date)
-            #     amount = ig.get_invoiceable_amount(ie)
-            #     if amount:
-            #         item.amount += amount
-            # # if not item.amount:
-            # #     dd.logger.info("20181113 amount is 0")
-            # #     continue
-            # n = len(item.preview.splitlines())
-            # if n <= ItemsByPlan.row_height:
-            #     if item.preview:
-            #         item.preview += '<br>\n'
-            #     ctx = dict(
-            #         title=obj.get_invoiceable_title(),
-            #         amount=item.amount,
-            #         currency=dd.plugins.ledger.currency_symbol)
-            #     item.preview += "{title} ({amount} {currency})".format(
-            #         **ctx)
-            # elif n == ItemsByPlan.row_height + 1:
-            #     item.preview += '...'
-
-
-            item.preview = "" # _("{} items").format(len(invoice_items))
+            # item.preview = "" # _("{} items").format(len(invoice_items))
             total_amount = ZERO
             for n, i in enumerate(invoice_items):
-                i.discount_changed()
+                # i.discount_changed()
                 total_amount += i.get_amount() or ZERO
                 # item.preview += "<br>\n"
                 # ctx = dict(
@@ -316,14 +320,13 @@ class Plan(UserPlan):
         # M = ITEM_MODEL._meta.get_field('voucher').remote_field.model
         kwargs.update(
             journal=self.area.journal,
-            entry_date=self.today,
-            invoicing_min_date=self.min_date,
-            invoicing_max_date=self.get_max_date())
+            entry_date=self.today)
+            # invoicing_min_date=self.min_date,
+            # invoicing_max_date=self.get_max_date())
         invoice = M(**kwargs)
         invoice.fill_defaults()
         return invoice
 
-
     toggle_selections = ToggleSelection()
 
     def __str__(self):
@@ -331,9 +334,6 @@ class Plan(UserPlan):
         # return self._meta.verbose_name
         return str(self.user)
 
-generator_label = _("Generator")
-
-
 
 class Item(dd.Model):
     class Meta:
@@ -344,22 +344,24 @@ class Item(dd.Model):
 
     plan = dd.ForeignKey('invoicing.Plan', related_name="items")
     partner = dd.ForeignKey('contacts.Partner')
+    # invoice_recipient = dd.ForeignKey(
+    #     'contacts.Partner', blank=True, null=True,
+    #     related_name="items_by_invoice_recipient")
 
     generator_type = dd.ForeignKey(
         ContentType,
         editable=True,
         blank=True, null=True,
-        verbose_name=format_lazy(u"{} {}", generator_label, _('(type)')))
+        verbose_name=format_lazy("{} {}", generator_label, _('(type)')))
     generator_id = GenericForeignKeyIdField(
         generator_type,
         editable=True,
         blank=True, null=True,
-        verbose_name=format_lazy(u"{} {}", generator_label, _('(object)')))
+        verbose_name=format_lazy("{} {}", generator_label, _('(object)')))
     generator = GenericForeignKey(
         'generator_type', 'generator_id',
         verbose_name=generator_label)
 
-
     # first_date = models.DateField(_("First date"))
     # last_date = models.DateField(_("Last date"))
     amount = dd.PriceField(_("Amount"), default=ZERO)
@@ -401,8 +403,9 @@ class Item(dd.Model):
                 generators = [
                     ig for ig in self.plan.get_generators_for_plan(
                         self.partner) if ig.allow_group_invoices()]
+                # assert len(generators) > 0
             for ig in generators:
-                info = ig.compute_invoicing_info(max_date)
+                info = ig.compute_invoicing_info(self.plan.min_date, max_date)
                 pt = ig.get_invoiceable_payment_term()
                 if pt:
                     invoice.payment_term = pt
@@ -410,8 +413,11 @@ class Item(dd.Model):
                 if pt:
                     invoice.paper_type = pt
 
+                ig.setup_invoice_from_suggestion(invoice, self.plan, info)
+
                 # for i in ig.get_invoice_items(info, ITEM_MODEL, ar):
                 for i in ig.get_invoice_items(info, invoice, ar):
+                    i.invoiceable = ig
                     # kwargs.update(voucher=invoice)
                     # i = ITEM_MODEL(**kwargs)
                     # if 'amount' in kwargs:
@@ -423,7 +429,7 @@ class Item(dd.Model):
 
         if len(items) == 0:
             # neither invoice nor items are saved
-            raise Warning(_("No invoiceables found for %s.") % self)
+            raise Warning(_("Nothing to invoice for %s.") % self)
             # dd.logger.warning(
             #     _("No invoiceables found for %s.") % self.partner)
             # return
@@ -432,6 +438,7 @@ class Item(dd.Model):
         invoice.save()
 
         for ig, i in items:
+            # assert i.voucher is None
             # assign voucher after it has been saved
             i.voucher = invoice
             ig.setup_invoice_item(i)
@@ -442,7 +449,7 @@ class Item(dd.Model):
 
             # title = i.title
             # i.product_changed()
-            i.discount_changed()
+            # i.discount_changed()
             # i.title = title
             i.full_clean()
             i.save()
@@ -464,7 +471,8 @@ class Item(dd.Model):
 class Plans(dd.Table):
     required_roles = dd.login_required(LedgerUser)
     model = "invoicing.Plan"
-    detail_layout = """user area today min_date max_date partner
+    detail_layout = """user area today min_date max_date
+    partner order
     invoicing.ItemsByPlan
     """
 
@@ -502,9 +510,6 @@ class InvoicingsByGenerator(dd.Table):
                    "unit_price total_incl #total_base #total_vat *"
 
 
-invoiceable_label = dd.plugins.invoicing.invoiceable_label
-
-
 dd.inject_field(
     'products.Product', 'tariff', dd.ForeignKey(
         'invoicing.Tariff', blank=True, null=True))
@@ -577,5 +582,7 @@ dd.inject_action(
 def install_start_action(sender=None, **kwargs):
     # vt = dd.plugins.invoicing.get_voucher_type()
     # vt.table_class.start_invoicing = StartInvoicingForJournal()
-
     rt.models.contacts.Partner.start_invoicing = StartInvoicingForPartner()
+    m = dd.plugins.invoicing.order_model
+    if m is not None:
+        m.start_invoicing = StartInvoicingForOrder()
