@@ -4,7 +4,7 @@
 """
 Adds functionality for managing products.
 
-See :doc:`/specs/products`.
+See :doc:`/plugins/products`.
 
 """
 
