@@ -1,10 +1,6 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2014-2018 Rumma & Ko Ltd
-# License: BSD (see file COPYING for details)
-
-
-from __future__ import unicode_literals
-from __future__ import print_function
+# Copyright 2014-2021 Rumma & Ko Ltd
+# License: GNU Affero General Public License v3 (see file COPYING for details)
 
 
 from lino.api import dd, pgettext, _
@@ -20,4 +16,3 @@ Priorities.add_item('20', pgettext("priority", "High"), 'high')
 Priorities.add_item('30', pgettext("priority", "Normal"), 'normal')
 Priorities.add_item('40', pgettext("priority", "Low"), 'low')
 Priorities.add_item('50', pgettext("priority", "Very Low"), 'very_low')
-
